﻿
################################################################################
#	功能：取代檔案中的內容
#	作者： Kuo yi-lun
# 電郵： logo@molerat.net
#	日期： 2020/02/16
#	版本： 0.3.2
#	更新： 2020/06/28
#	環境： win 10 1909 64bit + python 3.7.6 + wx 4.0.3
################################################################################

import wx
import wx.lib.scrolledpanel
import os
from chardet import detect

# 取代檔案中的內容程式類別
class ChangeCode(wx.Frame):
	def __init__(self, parent, title):
		# 不出現系統功能表但有標題列和關閉按鈕
		super(ChangeCode, self).__init__(parent, title = title, style = wx.CAPTION | wx.CLOSE_BOX)
		# 把框架放到一個函數中
		self.InitUI()
		# 在畫面中間顯示這個視窗
		self.Center()

	# 讓使用者進行操作的框架函數
	def InitUI(self):
		self.default_code = '' # 預設刪除的符號
		self.filename_paths = [] # 使用者選擇的所有路徑及檔名
		
		# 所有元件都在 panel 上
		panel = wx.lib.scrolledpanel.ScrolledPanel(self, wx.ID_ANY)
		# 宣告字體與大小
		font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.NORMAL, False, 'Courier New')
		# 設定 panel 字體與大小
		panel.SetFont(font)
		
		# 產生主要的 boxsizer 垂直 裡面會包元素
		self.main_sizer = wx.BoxSizer(wx.VERTICAL)
		# 原始內容 編輯區
		self.source_code_label = wx.StaticText(parent = panel, label = "原始內容")
		self.source_code_text = wx.TextCtrl(parent = panel)
		# 預設原始內容
		self.source_code_text.SetValue(self.default_code)
		self.main_sizer.Add(self.source_code_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.source_code_text, 1, wx.EXPAND | wx.ALL)
		# 取代內容 編輯區
		self.inst_code_label = wx.StaticText(parent = panel, label = "取代內容")
		self.inst_code_text = wx.TextCtrl(parent = panel)
		self.main_sizer.Add(self.inst_code_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.inst_code_text, 1, wx.EXPAND | wx.ALL)
		# 選擇檔案 按鈕 
		self.select_file_btn = wx.Button(parent = panel, label = "選擇檔案")
		self.Bind(wx.EVT_BUTTON, self.onSelectFileBtnClick, self.select_file_btn)
		self.main_sizer.Add(self.select_file_btn, 1, wx.ALL)
		# 檔名 編輯區
		self.filename_label = wx.StaticText(parent = panel, label = "輸入檔名")
		self.filename_text = wx.TextCtrl(parent = panel)
		self.main_sizer.Add(self.filename_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.filename_text, 1, wx.EXPAND | wx.ALL)
		# 送出 按鈕 
		self.send_btn = wx.Button(parent = panel, label = "送出")
		self.Bind(wx.EVT_BUTTON, self.onSendBtnClick, self.send_btn)
		self.main_sizer.Add(self.send_btn, 1, wx.ALL)
		# 版本說明
		self.readme_label = wx.StaticText(parent = panel, label = "關於")
		self.readme_text = wx.TextCtrl(parent = panel, style = wx.TE_READONLY + wx.TE_MULTILINE)
		self.readme_text.SetValue('版本： 0.3.2 作者： logo@molerat.net')
		self.main_sizer.Add(self.readme_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.readme_text, 1, wx.EXPAND | wx.ALL)
		
		# 加入主要 boxsizer 到 panel
		panel.SetSizer(self.main_sizer)
		# 當視窗大小改變時就自動重繪 panel 畫面佈局
		panel.SetAutoLayout(True)
		# 設定 panel 可以捲動
		panel.SetupScrolling()
	
	# 選擇檔案的方法
	def onSelectFileBtnClick(self,event):
		# wildcard 可以用來選擇可以開啟的檔案類型
		with wx.FileDialog(self, message = "選擇檔案", defaultDir = os.getcwd(), defaultFile = "", style = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_CHANGE_DIR | wx.FD_MULTIPLE) as file_dialog:
			# 選擇檔案失敗就結束該函數
			if file_dialog.ShowModal() == wx.ID_CANCEL:
				return
			# 取得使用者選擇的完整路徑加檔名
			self.filename_paths = file_dialog.GetPaths()
			multi_all_file_string = ','.join(self.filename_paths)
			# 把路徑放到檔名的編輯區
			self.filename_text.SetValue(multi_all_file_string)
			# 把焦點移到檔名編輯區
			self.filename_text.SetFocus()
	
	# 按下確定按鈕先取得資訊並做基本檢查才執行替代工作
	def onSendBtnClick(self,event):
		# 取得原始內容
		get_source_code = self.source_code_text.GetValue()
		# 取得替代內容
		get_inst_code = self.inst_code_text.GetValue()
		# 一定要從編輯區取得檔案列表，因為使用者可能用選的也可能用輸入的
		filenames = self.filename_text.GetValue().split(',')
		if self.hasNoError(get_source_code, get_inst_code, filenames):
			self.executeChange(get_source_code, get_inst_code, filenames)

	# 錯誤檢查
	def hasNoError(self, gsc, gic, fns):
		# 沒填原始內容
		if gsc == '':
			msg = f'原始內容為必填'
			self.dialog(msg, self.source_code_text, caption = '錯誤訊息對話')
			return False
		# 沒有選擇檔案，只需檢查路徑編輯區
		elif fns[0].strip() == '':
			msg = f'尚未選擇檔案'
			self.dialog(msg, self.filename_text, caption = '錯誤訊息對話')
			return False
		# 選擇的檔案不存在或不是檔案
		else:
			for fn in fns:
				if not os.path.isfile(fn.strip()):
					msg = f'{fn} 不存在或不是檔案'
					self.dialog(msg, self.filename_text, caption = '錯誤訊息對話')
					return False
		return True

	# 執行取代工作
	def executeChange(self, gsc, gic, fns):
		for fn in fns:
			# 去掉路徑檔名前後的空白或換行符號等
			fn = fn.strip()
			# 取得這個檔案的編碼
			coding = self.getCoding(fn)
			# 依據這個編碼來開檔並取出檔案內容
			content = self.getContent(fn, coding)
			# 取代檔案中的內容，回傳新內容、取代次數與出現的行號
			content, count, line_number = self.changeCode(content, gsc, gic)
			# 如果檔案中確實有出現需被取代的內容
			if count > 0:
				self.writeFile(fn, content, coding) # 將新的檔案內容取代舊的檔案內容
				line = ' '.join(line_number) # 把行號列表變成字串，中間用空格隔開
				# 刪除訊息
				if gic == '':
					msg = f'刪除完成，在 {fn} 刪掉了 {gsc} 共 {count} 個 分別出現在第 {line} 行'
				# 取代訊息
				else:
					msg = f'取代完成，在 {fn} 將 {gsc} 取代成 {gic} 共 {count} 個 分別出現在第 {line} 行'
			# 檔案沒有需要取代的內容
			else:
				msg = f'檔案 {fn} 沒有任何 {gsc}'
			# 呼叫顯示訊息對話框
			self.dialog(msg, self.send_btn, caption = '執行完成對話')

	# 顯示訊息對話框	
	def dialog(self, message, this_text, caption):
		with wx.MessageDialog(self, message, caption, wx.OK) as dlg:
			dlg.SetOKLabel("確定") # 自己設定中文敘述，預設值是 ok
			dlg.CenterOnScreen() # 設定在畫面中間顯示
			val = dlg.ShowModal() # 取得按下按鈕的值 wx.ID_OK wx.ID_CANCEL
			dlg.Show() # 顯示該對話框
			# 關閉對話框後將游標移到什麼元件上
			this_text.SetFocus()
	
	# 取得檔案內容的編碼
	def getCoding(self, fn):
		with open(r'' + fn, 'rb') as f:
			content = f.read()
			return detect(content)['encoding']
	
	# 依據編碼取得檔案內容
	def getContent(self, fn, coding):
		content = []
		with open(r'' + fn, 'r', encoding = coding) as f:
			for line in f:
				content.append(line)
		return content
	
	# 取代檔案中的內容
	def changeCode(self, content, source_code, inst_code):
		count = 0
		line_number = []
		for index, line in enumerate(content[:], 0):
			if source_code in line:
				count += line.count(source_code)
				line_number.append(str(index + 1))
				line = line.replace(source_code, inst_code)
				# 取代內容後，只剩空行就把空行刪掉
				if len(line.strip(' ')) == 1 and '\n' in line:
					line = line.strip()
				content[index] = line
		return content, count, line_number
	
	# 將更改後的檔案內容寫回原本的檔案
	def writeFile(self, fn, content, coding):
		# 把反斜線都改成斜線方便統一處理
		fn = fn.replace('\\', '/')
		# 如果是相對路徑就加上原本的工作資料夾路徑
		if '/' not in fn:
			fn = os.getcwd() + '/' + fn
		with open(fn, 'w', encoding = coding) as f:
			for line in content:
				f.write(line)
	
# 主函數建立類別並讓程式不斷巡環
def main():
	app = wx.App()
	change_code = ChangeCode(None, title = "取代檔案中的內容")
	change_code.Show()
	app.MainLoop()

# 程式進入點 呼叫主函數
if __name__ == '__main__':
	main()
