﻿
################################################################################
#	功能：刪除檔案中的特殊字元
#	作者： Kuo yi-lun
# 電郵： logo@forblind.org.tw
#	日期： 2020/02/16
#	版本： 0.2.2
#	更新： 2020/06/21
#	環境： win 10 1909 64bit + python 3.7.6 + wx 4.0.3
################################################################################

import wx
import wx.lib.scrolledpanel
import os
from chardet import detect

# 刪除特殊字元程式的類別
class DelCode(wx.Frame):
	def __init__(self, parent, title):
		# 不出現系統功能表但有標題列和關閉按鈕
		super(DelCode, self).__init__(parent, title = title, style = wx.CAPTION | wx.CLOSE_BOX)
		# 把框架放到一個函數中
		self.InitUI()
		# 在畫面中間顯示這個視窗
		self.Center()

	# 讓使用者進行操作的框架函數
	def InitUI(self):
		self.default_code = '' # 預設刪除的符號
		self.filename_paths = [] # 使用者選擇的所有路徑及檔名
		
		# 所有元件都在 panel 上
		panel = wx.lib.scrolledpanel.ScrolledPanel(self, wx.ID_ANY)
		# 宣告字體與大小
		font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.NORMAL, False, 'Courier New')
		# 設定 panel 字體與大小
		panel.SetFont(font)
		
		# 產生主要的 boxsizer 垂直 裡面會包元素
		self.main_sizer = wx.BoxSizer(wx.VERTICAL)
		# 欲刪除的字元 編輯區
		self.code_label = wx.StaticText(parent = panel, label = "輸入欲刪除之字元")
		self.code_text = wx.TextCtrl(parent = panel)
		# 預設刪除字元
		self.code_text.SetValue(self.default_code)
		self.main_sizer.Add(self.code_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.code_text, 1, wx.EXPAND | wx.ALL)
		# 檔名 編輯區
		self.filename_label = wx.StaticText(parent = panel, label = "輸入檔名")
		self.filename_text = wx.TextCtrl(parent = panel)
		self.main_sizer.Add(self.filename_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.filename_text, 1, wx.EXPAND | wx.ALL)
		# 選擇檔案 按鈕 
		self.select_file_btn = wx.Button(parent = panel, label = "選擇檔案")
		self.Bind(wx.EVT_BUTTON, self.onSelectFileBtnClick, self.select_file_btn)
		self.main_sizer.Add(self.select_file_btn, 1, wx.ALL)
		# 確定 按鈕 
		self.send_btn = wx.Button(parent = panel, label = "確定")
		self.Bind(wx.EVT_BUTTON, self.onSendBtnClick, self.send_btn)
		self.main_sizer.Add(self.send_btn, 1, wx.ALL)
		# 版本說明
		self.readme_label = wx.StaticText(parent = panel, label = "版本")
		self.readme_text = wx.TextCtrl(parent = panel, style = wx.TE_READONLY + wx.TE_MULTILINE)
		self.readme_text.SetValue('0.2.2')
		self.main_sizer.Add(self.readme_label, 1, wx.EXPAND | wx.ALL)
		self.main_sizer.Add(self.readme_text, 1, wx.EXPAND | wx.ALL)
		
		# 加入主要 boxsizer 到 panel
		panel.SetSizer(self.main_sizer)
		# 當視窗大小改變時就自動重繪 panel 畫面佈局
		panel.SetAutoLayout(True)
		# 設定 panel 可以捲動
		panel.SetupScrolling()
	
	# 選擇檔案的方法
	def onSelectFileBtnClick(self,event):
		# wildcard 可以用來選擇可以開啟的檔案類型
		with wx.FileDialog(self, message = "選擇檔案", defaultDir = os.getcwd(), defaultFile = "", style = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_CHANGE_DIR | wx.FD_MULTIPLE) as file_dialog:
			# 選擇檔案失敗就結束該函數
			if file_dialog.ShowModal() == wx.ID_CANCEL:
				return
			# 取得使用者選擇的完整路徑加檔名
			self.filename_paths = file_dialog.GetPaths()
			multi_all_file_string = ','.join(self.filename_paths)
			# 把路徑放到檔名的編輯區
			self.filename_text.SetValue(multi_all_file_string)
			# 把焦點移到檔名編輯區
			self.filename_text.SetFocus()
	
	# 按下確定按鈕所做的事，程式的主要流程控制
	def onSendBtnClick(self,event):
		# 取得使用者想要刪除的字元或符號
		code = self.code_text.GetValue()
		# 檔案不存在就報錯
		for filename in self.filename_paths:
			if not os.path.isfile(filename):
				self.dialog(f'{filename} 不存在或不是檔案', self.filename_text)
			else:
				# 取得這個檔案的編碼
				coding = self.getCoding(filename)
				# 依據這個編碼來開檔並取出檔案內容
				content = self.getContent(filename, coding)
				# 刪除檔案中的特殊字元，回傳新內容、刪除字元數與出現的行號
				content, count, line_number = self.deleteCode(content, code)
				# 如果檔案中確實有出現該字元
				if count > 0:
					self.writeFile(filename, content, coding) # 將更改後的檔案內容寫到另一個檔案中，原檔案都不動
					line = ' '.join(line_number) # 把行號列表變成字串，中間用空格隔開
					msg = f'執行成功，在 {filename} 刪了 {count} 個 {code} 分別出現在第 {line} 行'
				# 檔案沒有需要刪除的字元
				else:
					msg = f'檔案 {filename} 沒有任何 {code}'
				# 不管如何都要顯示對話框訊息讓使用者知道結果
				self.dialog(msg, self.filename_text)
	
	# 顯示訊息對話框	
	def dialog(self, message, this_text, caption = "刪除特殊字元對話框"):
		with wx.MessageDialog(self, message, caption, wx.OK) as dlg:
			dlg.SetOKLabel("確定") # 自己設定中文敘述，預設值是 ok
			dlg.CenterOnScreen() # 設定在畫面中間顯示
			val = dlg.ShowModal() # 取得按下按鈕的值 wx.ID_OK wx.ID_CANCEL
			dlg.Show() # 顯示該對話框
			# 關閉對話框後將游標移到什麼元件上
			this_text.SetFocus()
	
	# 取得檔案內容的編碼
	def getCoding(self, fn):
		with open(r'' + fn, 'rb') as f:
			content = f.read()
			return detect(content)['encoding']
	
	# 依據編碼取得檔案內容
	def getContent(self, fn, coding):
		content = []
		with open(r'' + fn, 'r', encoding = coding) as f:
			for line in f:
				content.append(line)
		return content
	
	# 刪除檔案中的特殊字元
	def deleteCode(self, content, code):
		count = 0
		line_number = []
		for index, line in enumerate(content[:], 0):
			print(line)
			if code in line:
				count += line.count(code)
				line_number.append(str(index + 1))
				line = line.replace(code, '')
				# 刪除特殊符號後只剩空行就把空行刪掉
				if len(line.strip(' ')) == 1 and '\n' in line:
					line = line.strip()
				content[index] = line
		return content, count, line_number
	
	# 將更改後的檔案內容寫到另一個檔案中，原檔案都不動
	def writeFile(self, fn, content, coding):
		# 把反斜線都改成斜線方便統一處理
		fn = fn.replace('\\', '/')
		# 如果是相對路徑就加上原本的工作資料夾路徑
		if '/' not in fn:
			fn = os.getcwd() + '/' + fn
		path = fn.split('/')[:-1]
		path = '/'.join(path)
		tail = fn.split('/')[-1]
		tail = 'x' + ' ' + tail
		new_filename = path + '/' + tail
		with open(new_filename, 'w', encoding = coding) as f:
			for line in content:
				f.write(line)
	
# 主函數建立類別並讓程式不斷巡環
def main():
	app = wx.App()
	del_code = DelCode(None, title = "刪除檔案中的字元")
	del_code.Show()
	app.MainLoop()

# 程式進入點 呼叫主函數
if __name__ == '__main__':
	main()
